@@ -10,9 +10,7 @@ pub struct EdPatch {
 impl crate::ContentPatch for EdPatch {
     fn apply_exact(&self, orig: &[u8]) -> Result<Vec<u8>, crate::ApplyError> {
         let lines = splitlines(orig).collect::<Vec<_>>();
-        let result = self
-            .apply(&lines)
-            .map_err(|e| crate::ApplyError::Conflict(e))?;
+        let result = self.apply(&lines).map_err(crate::ApplyError::Conflict)?;
         Ok(result)
     }
 }
@@ -27,7 +25,7 @@ impl EdPatch {
                     assert_eq!(start, end);
                     let existing = match data.get(start - 1) {
                         Some(existing) => existing,
-                        None => return Err(format!("line {} does not exist", start).into()),
+                        None => return Err(format!("line {} does not exist", start)),
                     };
                     if existing != expected {
                         return Err(format!(
